from abc import abstractmethod

from checkov.terraform.checks.data.registry import data_registry
from checkov.terraform.checks.utilities.base_check import BaseCheck


class BaseDataCheck(BaseCheck):
    def __init__(self, name, id, categories, supported_data):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_data,
                         block_type="data")
        self.supported_data = supported_data
        data_registry.register(self)

    @abstractmethod
    def scan_data_conf(self, conf):
        raise NotImplementedError()

    def scan_entity_conf(self, conf):
        return self.scan_data_conf(conf)



import logging
from abc import ABC, abstractmethod

from checkov.terraform.models.enums import CheckResult


class BaseCheck(ABC):
    id = ""
    name = ""
    categories = []
    supported_entities = []

    def __init__(self, name, id, categories, supported_entities, block_type):
        self.name = name
        self.id = id
        self.categories = categories
        self.block_type = block_type
        self.supported_entities = supported_entities
        self.logger = logging.getLogger("{}".format(self.__module__))

    def run(self, scanned_file, entity_configuration, entity_name, entity_type, skip_info):
        check_result = {}
        if skip_info:
            check_result['result'] = CheckResult.SKIPPED
            check_result['suppress_comment'] = skip_info['suppress_comment']
            message = "File {}, {} \"{}.{}\" check \"{}\" Result: {}, Suppression comment: {} ".format(
                scanned_file, self.block_type, entity_type,
                entity_name,
                self.name,
                check_result, check_result['suppress_comment'])
        else:
            try:
                check_result['result'] = self.scan_entity_conf(entity_configuration)
                message = "File {}, {}  \"{}.{}\" check \"{}\" Result: {} ".format(scanned_file, self.block_type,
                                                                                   entity_type,
                                                                                   entity_name,
                                                                                   self.name,
                                                                                   check_result)
                self.logger.debug(message)

            except Exception as e:
                self.logger.error(
                    "Failed to run check {} for configuration {} ".format(self.name, str(entity_configuration)))
                raise e
        return check_result

    @abstractmethod
    def scan_entity_conf(self, conf):
        raise NotImplementedError()



from checkov.terraform.checks.resource.base_check import BaseResourceCheck
from checkov.terraform.models.enums import CheckResult, CheckCategories

class GoogleContainerClusterClientCertificateEnabled(BaseResourceCheck):
    def __init__(self):
        name = "Ensure a client certificate is used by clients to authenticate to Kubernetes Engine Clusters"
        id = "CKV_GCP_13"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for client certificate configuration on google_container_cluster:
            https://www.terraform.io/docs/providers/google/r/container_cluster.html#client_certificate_config
        :param conf: google_container_cluster configuration
        :return: <CheckResult>
        """
        if 'master_auth' in conf and 'client_certificate_config' in conf['master_auth'][0]:
            if 'issue_client_certificate' in conf['master_auth'][0]['client_certificate_config'][0]:
                if conf['master_auth'][0]['client_certificate_config'][0]['issue_client_certificate'][0]:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = GoogleContainerClusterClientCertificateEnabled()
